/**

  Spago4Q - The Business Intelligence Free Platform

  Copyright (C) 2009 Engineering Ingegneria Informatica S.p.A.

  Spago4Q is free software; you can redistribute it and/or
  modify it under the terms of the GNU Lesser General Public
  License as published by the Free Software Foundation; either
  version 2.1 of the License, or (at your option) any later version.

  Spago4Q is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  Lesser General Public License for more details.

  You should have received a copy of the GNU Lesser General Public
  License along with this library; if not, write to the Free Software
  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

**/
package it.eng.spago4q.extractors;

import it.eng.spago.error.EMFUserError;
import it.eng.spago4q.bo.DataSourceParameter;
import it.eng.spago4q.bo.EOperation;
import it.eng.spago4q.bo.EOperationParameter;
import it.eng.spago4q.dao.DAOFactory;
import it.eng.spago4q.dao.IES4QFKDAO;
import it.eng.spago4q.dao.ILogDAO;
import it.eng.spago4q.extractors.bo.GenericItemInterface;

import java.util.Date;
import java.util.Iterator;
import java.util.List;

public abstract class AbstractExtractor implements ExtractorInterface {

	private EOperation eOperation = null;
	private List operationParameter = null;
	private List dataSourceParameter = null;
	private static final String EOPERATIONPARAMETER_DAO = "EOperationParameterDAO";
	private static final String EDATASOURCEPARAMETER_DAO = "EDataSourceParameterDAO";
	private static final String ELOG_DAO = "ELogDAO";

	public List<GenericItemInterface> execute() throws EMFUserError {
		List<GenericItemInterface> toReturn = null;
		setUp();
		try {
			toReturn = extract();
		} catch (EMFUserError emfUE) {
			throw emfUE;
		} finally {
			tearDown();
		}
		return toReturn;
	}

	public void init(final EOperation operation) throws EMFUserError {
		this.eOperation = operation;
		setDataSourceParameter(operation.getDataSource().getId());

		setOperationParameter(operation.getId());
	}

	protected EOperation getEOperation() {
		return eOperation;
	}

	public List getOperationParameter() {
		return operationParameter;
	}

	public void setOperationParameter(final Integer operationId) throws EMFUserError {
		IES4QFKDAO operationParameterDAO = (IES4QFKDAO) DAOFactory
				.getDAO(EOPERATIONPARAMETER_DAO);
		operationParameter = operationParameterDAO.loadObjectList(operationId,
				null, null);
	}

	public List getDataSourceParameter() {
		return dataSourceParameter;
	}

	public void setDataSourceParameter(final Integer dataSourceId)
			throws EMFUserError {
		IES4QFKDAO dataSourceParameterDAO = (IES4QFKDAO) DAOFactory
				.getDAO(EDATASOURCEPARAMETER_DAO);
		dataSourceParameter = dataSourceParameterDAO.loadObjectList(
				dataSourceId, null, null);
	}

	protected String readDataSourceParameterValue(final String key) {
		String toReturn = null;
		Iterator it = (Iterator) getDataSourceParameter().iterator();
		while (it.hasNext()) {
			DataSourceParameter currentDataSourceParameter = (DataSourceParameter) it
					.next();
			if (currentDataSourceParameter.getName().equals(key)){
				toReturn = currentDataSourceParameter.getValue();
			}
		}
		return toReturn;
	}

	protected String readOperationParameterValue(final String key) {
		String toReturn = null;
		Iterator it = (Iterator) getOperationParameter().iterator();
		while (it.hasNext()) {
			EOperationParameter currentOperationParameter = (EOperationParameter) it
					.next();
			if (currentOperationParameter.getName().equals(key)){
				toReturn = currentOperationParameter.getValue();
			}
		}
		return toReturn;
	}

	protected abstract void setUp();

	protected abstract void tearDown();

	protected abstract List<GenericItemInterface> extract() throws EMFUserError;

	public String dataSourceTest(final Integer dataSourceId) throws EMFUserError {
		return "No data source test found!";
	}

	public String operationTest(final Integer operationId) throws EMFUserError {
		return "No operation test found!";
	}

	public Date getLastExtractionDate() throws EMFUserError{
		Date toReturn = null;
		if (eOperation != null){
			ILogDAO logDAO = (ILogDAO)DAOFactory.getDAO(ELOG_DAO);
			toReturn = logDAO.getLastOperationExecutionDate(eOperation.getId());
		}
		return toReturn;
	}
}
