/**

  Spago4Q - The Business Intelligence Free Platform

  Copyright (C) 2009 Engineering Ingegneria Informatica S.p.A.

  Spago4Q is free software; you can redistribute it and/or
  modify it under the terms of the GNU Lesser General Public
  License as published by the Free Software Foundation; either
  version 2.1 of the License, or (at your option) any later version.

  Spago4Q is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  Lesser General Public License for more details.

  You should have received a copy of the GNU Lesser General Public
  License along with this library; if not, write to the Free Software
  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

**/
package it.eng.spago4q.extractors;

import it.eng.spago.error.EMFUserError;
import it.eng.spago4q.bo.ELog;
import it.eng.spago4q.bo.EOperation;
import it.eng.spago4q.dao.DAOFactory;
import it.eng.spago4q.dao.IES4QFKDAO;
import it.eng.spago4q.extractors.bo.GenericItemInterface;
import it.eng.spago4q.extractors.bo.InterfaceItem;
import it.eng.spago4q.extractors.script.ScriptExecutor;
import it.eng.spago4q.extractors.xml.FileImportExtractor;

import java.util.Date;
import java.util.Iterator;
import java.util.List;

public class FileImportCoordinator {

	private FileImportExtractor fileImportExtractor;
	private EOperation operation;
	private Date operationDate;

	public void init(final Integer operationId, final String fileName,
			final String xPath, final Date operationDatePar) throws EMFUserError {
		fileImportExtractor = new FileImportExtractor();
		fileImportExtractor.init(fileName, xPath);
		IES4QFKDAO eOperationDAO = (IES4QFKDAO) DAOFactory
				.getDAO("EOperationDAO");
		operation = (EOperation) eOperationDAO.loadObjectById(operationId);
		this.operationDate = operationDatePar;

	}

	public void run() throws EMFUserError {
		List<GenericItemInterface> genericItemList = fileImportExtractor
				.execute();

		ScriptExecutor scriptExecutor = new ScriptExecutor();
		DomainValueCacheCreator domainValueCacheCreator = new DomainValueCacheCreator(
				operation.getInterfaceType().getId());

		ELog eLog = new ELog();
		eLog.setProcessId(null);
		eLog.setOperationId(operation.getId());
		eLog.setExtracted(genericItemList.size());
		eLog.setOperationDate(operationDate);
		eLog.setExecutionDate(new Date());

		IES4QFKDAO eLogDao = (IES4QFKDAO) DAOFactory.getDAO("ELogDAO");

		Integer logId = eLogDao.insertObject(eLog);

		eLog.setIdLog(logId);

		List<InterfaceItem> interfaceItem = MapMaker.mapping(operation,
				genericItemList, scriptExecutor, domainValueCacheCreator);

		FactTableManager factTableManager = new FactTableManager(operation
				.getInterfaceType().getId());
		int numberOfInsert = 0;

		for (Iterator iterator2 = interfaceItem.iterator(); iterator2.hasNext();) {
			InterfaceItem interfaceItem2 = (InterfaceItem) iterator2.next();
			if (factTableManager.loadData(operationDate, operation
					.getInterfaceType().getId(), interfaceItem2, logId)) {
				numberOfInsert++;
			}
		}
		eLog.setInserted(numberOfInsert);
		eLogDao.modifyObject(eLog);
	}
}
