/**

  Spago4Q - The Business Intelligence Free Platform

  Copyright (C) 2009 Engineering Ingegneria Informatica S.p.A.

  Spago4Q is free software; you can redistribute it and/or
  modify it under the terms of the GNU Lesser General Public
  License as published by the Free Software Foundation; either
  version 2.1 of the License, or (at your option) any later version.

  Spago4Q is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  Lesser General Public License for more details.

  You should have received a copy of the GNU Lesser General Public
  License along with this library; if not, write to the Free Software
  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

**/
package it.eng.spago4q.extractors.utilities;

import it.eng.spago.error.EMFErrorSeverity;
import it.eng.spago.error.EMFUserError;
import it.eng.spago4q.bo.EOperation;
import it.eng.spago4q.extractors.ExtractorInterface;
import it.eng.spago4q.extractors.bo.GenericItemInterface;

import java.util.List;

import org.apache.log4j.Logger;

public final class OperationUtil {

	private static Logger logger = Logger.getLogger(OperationUtil.class);

	private OperationUtil() {
	}
	
	public static List<GenericItemInterface> executeOperation(
			final EOperation operation) throws EMFUserError {
		List<GenericItemInterface> toReturn = null;

		String extractorClassName = operation.getDataSource().getSourceType()
				.getExtractorClass();

		ExtractorInterface extractor = null;

		try {
			extractor = (ExtractorInterface) Class.forName(extractorClassName)
					.newInstance();
			extractor.init(operation);
			// extracted
			toReturn = extractor.execute();

		} catch (InstantiationException e) {
			logger.error("Error while Instantiate class "
					+ extractorClassName, e);
			throw new EMFUserError(EMFErrorSeverity.ERROR, 101);
		} catch (IllegalAccessException e) {
			logger.error("Error while Instantiate class "
					+ extractorClassName, e);
			throw new EMFUserError(EMFErrorSeverity.ERROR, 101);
		} catch (ClassNotFoundException e) {
			logger.error("Error while Instantiate class "
					+ extractorClassName, e);
			throw new EMFUserError(EMFErrorSeverity.ERROR, 101);
		}

		return toReturn;
	}

}
