/**

  Spago4Q - The Business Intelligence Free Platform

  Copyright (C) 2009 Engineering Ingegneria Informatica S.p.A.

  Spago4Q is free software; you can redistribute it and/or
  modify it under the terms of the GNU Lesser General Public
  License as published by the Free Software Foundation; either
  version 2.1 of the License, or (at your option) any later version.

  Spago4Q is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  Lesser General Public License for more details.

  You should have received a copy of the GNU Lesser General Public
  License along with this library; if not, write to the Free Software
  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

**/
package it.eng.spago4q.einterface.utils;

import it.eng.spago.base.SourceBean;
import it.eng.spago.error.EMFUserError;
import it.eng.spago4q.bo.InterfaceType;
import it.eng.spago4q.extractors.FactTableManager;
import it.eng.spago4q.source.utils.TestMessage;
import it.eng.spago4q.utilities.DetailS4Q1ObjectUtil;

public class DetailEInterfaceTypeUtil extends DetailS4Q1ObjectUtil {

	private static final String DAO_NAME = "EInterfaceTypeDAO";
	private static final String OBJ_NAME = "EInterfaceType";
	private static final String CREATE_TABLE = "CREATE";
	private static final String DROP_TABLE = "DROP";

	private TestMessage message;

	@Override
	protected String getDAOName() {
		return DAO_NAME;
	}

	@Override
	protected Object getObjectFromRequest(SourceBean serviceRequest) {
		InterfaceType toReturn = new InterfaceType();
		String name = (String) serviceRequest.getAttribute("name");
		String description = (String) serviceRequest
				.getAttribute("description");
		String tableName = (String) serviceRequest.getAttribute("tablename");
		String sDataSourceId = (String) serviceRequest
				.getAttribute("DATASOURCE");
		Integer dataSourceId = null;

		if (sDataSourceId != null && (!sDataSourceId.equals("-1"))) {
			dataSourceId = Integer.parseInt(sDataSourceId);
		}
		toReturn.setName(name);
		toReturn.setDescription(description);
		toReturn.setTableName(tableName);
		toReturn.setDataSourceId(dataSourceId);
		return toReturn;
	}

	@Override
	public void updateObjectFromRequest(SourceBean serviceRequest, Integer id)
			throws Exception {
		super.updateObjectFromRequest(serviceRequest, id);
		String tableOperation = (String) serviceRequest
				.getAttribute("TABLEOPERATION");
		if (tableOperation != null
				&& tableOperation.toUpperCase().equals(CREATE_TABLE)) {
			// create or alter table
			try {
				FactTableManager factTableManager = new FactTableManager(id);
				factTableManager.saveFactTable(id);
				message = new TestMessage(true, "FT.create.ok");
			} catch (EMFUserError e) {
				message = new TestMessage(false, "FT.create.error");
				throw e;
			}
		}
		if (tableOperation != null
				&& tableOperation.toUpperCase().equals(DROP_TABLE)) {
			// drop table
			try {
				FactTableManager factTableManager = new FactTableManager(id);
				factTableManager.dropFactTable(id);
				message = new TestMessage(true, "FT.drop.ok");
			} catch (EMFUserError e) {
				message = new TestMessage(false, "FT.drop.error");
				throw e;
			}
		}
	}

	@Override
	protected String getObjectName() {
		return OBJ_NAME;
	}

	@Override
	protected void setObjectId(Object object, Integer id) {
		((InterfaceType) object).setId(id);
	}

	@Override
	public void selectObject(Integer id, SourceBean serviceResponse)
			throws Exception {
		super.selectObject(id, serviceResponse);
		if (message != null){
			serviceResponse.setAttribute("MESSAGETOSHOW", message);
		}
	}
}
