/**

  Spago4Q - The Business Intelligence Free Platform

  Copyright (C) 2009 Engineering Ingegneria Informatica S.p.A.

  Spago4Q is free software; you can redistribute it and/or
  modify it under the terms of the GNU Lesser General Public
  License as published by the Free Software Foundation; either
  version 2.1 of the License, or (at your option) any later version.

  Spago4Q is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  Lesser General Public License for more details.

  You should have received a copy of the GNU Lesser General Public
  License along with this library; if not, write to the Free Software
  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

**/
package it.eng.spago4q.tools.utils;

import it.eng.spago.base.SourceBean;
import it.eng.spago.configuration.ConfigSingleton;
import it.eng.spago.error.EMFErrorHandler;
import it.eng.spago.error.EMFErrorSeverity;
import it.eng.spago.error.EMFUserError;
import it.eng.spago.validation.EMFValidationError;
import it.eng.spagobi.commons.utilities.GeneralUtilities;

import java.io.BufferedWriter;
import java.io.File;
import java.io.FileWriter;
import java.io.IOException;
import java.util.Date;

import org.apache.commons.fileupload.FileItem;
import org.apache.log4j.Logger;

public class FileImportUtil {
	private static final Logger logger = Logger.getLogger(FileImportUtil.class);
	private static final int MAX_DEFAULT_IMPORT_FILE_SIZE = 5242880;
	// saveFile
	public static String importFile(SourceBean serviceRequest,
			EMFErrorHandler emfErrorHandler) throws EMFUserError {
		String toReturn = null;
		byte[] content = readFile(serviceRequest, emfErrorHandler);
		toReturn = saveFile(content);
		return toReturn;
	}

	private static byte[] readFile(SourceBean serviceRequest,
			EMFErrorHandler emfErrorHandler) {
		byte[] content = null;
		String fileName = null;
		// lettura file da request a byte[]
		FileItem uploaded = (FileItem) serviceRequest
				.getAttribute("UPLOADED_FILE");

		if (uploaded != null) {
			fileName = GeneralUtilities
					.getRelativeFileNames(uploaded.getName());
			if (uploaded.getSize() == 0) {
				EMFValidationError error = new EMFValidationError(
						EMFErrorSeverity.ERROR, "uploadFile", "201");
				emfErrorHandler.addError(error);
			}
			int maxSize = getImportFileMaxSize();
			if (uploaded.getSize() > maxSize) {
				EMFValidationError error = new EMFValidationError(
						EMFErrorSeverity.ERROR, "uploadFile", "202");
				emfErrorHandler.addError(error);
			}
			if (uploaded.getSize() > 0) {
				try {
					content = uploaded.get();
				} catch (Exception e) {
					e.printStackTrace();
				}
			}
		}
		return content;
	}

	private static String saveFile(byte[] content) throws EMFUserError {
		String toReturn = null;
		if (content != null) {
			try {
				File tempDirectory = new File(getImportTempFolderPath());
				tempDirectory.mkdir();
				Date date = new Date();
				File file = new File(tempDirectory,"s4qimport"+ date.getTime()+".xml"); 
				// Create file if it does not exist
				boolean success = file.createNewFile();
				if (success) {
					toReturn = file.getAbsolutePath();
					BufferedWriter out = new BufferedWriter(
							new FileWriter(file));
					out.write(new String(content));
					out.close();

				} else {
					throw new EMFUserError("ERROR","Error while create the temporary file");
				}
			} catch (IOException e) {
				throw new EMFUserError("ERROR","Error while create the temporary file");
			}
		}
		return toReturn;
	}

	public static boolean deleteFile(String fileName) {
		boolean success = (new File(fileName)).delete();
		return success;
	}
	
	private static int getImportFileMaxSize() {
		logger.debug("IN");
		int toReturn = MAX_DEFAULT_IMPORT_FILE_SIZE;
		try {
			ConfigSingleton serverConfig = ConfigSingleton.getInstance();
			SourceBean maxSizeSB = (SourceBean) serverConfig.getAttribute("SPAGO4Q.IMPORT.IMPORT_FILE_MAX_SIZE");
			if (maxSizeSB != null) {
				String maxSizeStr = (String) maxSizeSB.getCharacters();
				logger.debug("Configuration found for max import file size: " + maxSizeStr);
				Integer maxSizeInt = new Integer(maxSizeStr);
				toReturn = maxSizeInt.intValue();
			} else {
				logger.debug("No configuration found for max import file size");
			}
		} catch (Exception e) {
			logger.error("Error while retrieving max import file size", e);
			logger.debug("Considering default value " + MAX_DEFAULT_IMPORT_FILE_SIZE);
			toReturn = MAX_DEFAULT_IMPORT_FILE_SIZE;
		}
		logger.debug("OUT: max size = " + toReturn);
		return toReturn;
	}

	private static String getImportTempFolderPath() {
		logger.debug("IN");
		String toReturn = null;
		try {
			ConfigSingleton serverConfig = ConfigSingleton.getInstance();
			SourceBean importerSB = (SourceBean) serverConfig.getAttribute("SPAGO4Q.IMPORT.IMPORT_TEMP_FOLDER");
			toReturn = (String) importerSB.getCharacters();
			toReturn = GeneralUtilities.checkForSystemProperty(toReturn);
			if (!toReturn.startsWith("/") && toReturn.charAt(1) != ':') {
				String root = ConfigSingleton.getRootPath();
				toReturn = root + "/" + toReturn;
			}
		} catch (Exception e) {
			logger.error("Error while retrieving export temporary folder path", e);
		} finally {
			logger.debug("OUT: export temporary folder path = " + toReturn);
		}
		return toReturn;
	}
}
